        program ccf_v4
c====================================================================
c	derive cross-correlation function in frequency domain
c
c       If this code helps your research a lot, please cite
c       1) Chen, K. X., H. Kuo-Chen*, D. Brown, Q. Li, Z. Ye, W. T. Liang, C. Y. Wang, and H. Yao (2016), Three-dimensional ambient noise tomography across the Taiwan Strait: The structure of a magma-poor rifted margin, Tectonics., 35, 8, 1782-1792, doi: 10.1002/2015TC004097
c       2) Chen, K. X., Y. Gung, B. Y. Kuo*, and T. Y. Huang (2018), Crustal magmatism and deformation fabrics in northeast Japan revealed by ambient noise tomography, Journal of Geophysical Research: Solid Earth, C123, doi:10.1029/2017JB015209 
c
c       08/13/2012 The 1st version was coded by Wen-Tzong Liang
c
c       07/06/2011 by Li-Wei Chen:
c       input a SAC file whose response has been removed!
c
c       10/17/2014 modify 1st by Kai-Xun Chen
c       1.  adjust order of input sac1 & sac2   
c           Ex: NACB.sac ANPB.sac NACB-ANPB.CCF
c       2.  add option for whitening, 0 for no_whiten
c       3.  add option for taper in frequency domain when doing whiten
c           aviod long peroid coherence ,unwanted signal
c           it will do taper for five times
c           0: no_taper, 0.1: taper for lower than 0.1 Hz 
c
c       12/22/2018 modify 2nd by Kai-Xun Chen
c       1.  input a list of daily sac-file in the same day,
c           read all the daily.sac, it will largely reduce the times of I/O.
c           Then, calculate CCFs according to the order below
c           1-2,1-3,1-4,1-5, 2-3,2-4,2-5, 3-4,3-5, 4-5
c           * Run zr01_List4CCF.csh to get the input/output list
c       2.  nn: how many npts in daily sac-file
c           ns: how many stations
c           be careful on the lenght of character      
c           timelen: half window of CCF
c           ovlp: percentage of overlapping
c           * timeline & ovlp will largely dominate the calculating time
c
c       11/18/2019      modify 3rd by Kai-Xun Chen
c       1.  allocatable parameters for some of the variables
c       2.  option for replace the old CCFs
c
c       07/08/2020      modify 4th by Kai-Xun Chen
c       1.  openmp is used to speed up the procedure
c       2.  option for ACF(A), CCF(C), or Both(B)
c
c       08/07/2020      modify 4th by Kai-Xun Chen
c       1.  fix a minor bug
c====================================================================
        use OMP_LIB
        parameter (nn=172800)
        character*100   sac_list, ccf_list, header
        character*100,allocatable:: inpsac(:)
        character*8     inp3,inp4,inp5
        character*10,allocatable:: kstnm(:),kcmpnm(:)
        character*100 :: buffer
        character*50    outccf(nn),kstnm0,kstnm1
        character*1     replace,abc
        logical         alive
        real*4,allocatable:: stla(:),stlo(:) ! read from input sacfile
        real*8 stla1d,stlo1d,stla2d,stlo2d ! for computing distance
        real,allocatable:: yy(:,:)
c        DIMENSION yy(ns,nn)
        real yall(nn),yyc(nn),y1(nn),y2(nn),yz(nn)!,yy
        real std1,std2,crt_std,maxv1,maxv2,crt1,crt2,y1std,y2std!,whiten
        real timelen
        real*8 dist,az,baz
        integer o,ij
        integer, parameter :: fileid1 = 10, fileid2 = 20
        integer :: status = 0
        integer,allocatable:: ni(:),nj(:)
  
        narg=IArgC()
        if(narg .ne. 4) then
          print *
          print *,'ccf_v3 is used to compute the cross-correlation function!'
          print *
          print *,'Usage: ccf_v3 [sac_list] [ccf_list] [ACF(A)/CCF(C)/Both(B)] [replace(Y/N)]'
          print *,'  Ex.: ccf_v3  sac_list   ccf_list         B                     Y'
          print *
          print *,'     replace = Y, the exist CCFs will be replace with the new one'
          stop
        endif  

        call GetArg(1,sac_list)
        call GetArg(2,ccf_list)
        call GetArg(3,inp3)
        read(inp3,*) abc
        call GetArg(4,inp4)
        read(inp4,*) replace
c        read(inp4,*) whiten
c        call GetArg(5,inp5)
c        read(inp5,*) replace


        open(fileid1,file=ccf_list)
        n = 1
        do while(.true.)
          read(unit=fileid1,fmt="(A100)",iostat=status) outccf(n)
          if (status /= 0 ) exit
          n = n + 1
        enddo
        o=n-1

        open(fileid2,file=sac_list)
        n = 1
        do while(.true.)
          read(unit=fileid2,fmt="(A100)",iostat=status) buffer
          if (status /= 0 ) exit
          n = n + 1
        enddo
        n=n-1
        close(fileid2)
      
        allocate(inpsac(n))
        allocate(kstnm(n))
        allocate(kcmpnm(n))
        allocate(stla(n))
        allocate(stlo(n))
        allocate(yy(n,nn))

        nij=0
        do i=1,n
          nij=nij+i
        enddo
        allocate(ni(nij),nj(nij))
        print *,'          Input:'
        open(fileid2,file=sac_list)
        do i=1,n
          read(unit=fileid2,fmt="(A100)",iostat=status) buffer
          call rsac1(buffer,yy(i,:),np,b,dt,nn,nerr)
          call getkhv('KSTNM',kstnm(i),nerr)
          call getkhv('KCMPNM',kcmpnm(i),nerr)
          call getfhv('STLA',stla(i),nerr)
          call getfhv('STLO',stlo(i),nerr)
          print *,i,buffer,stla(i),stlo(i)
        enddo
        close(fileid2)
        print *,' '
        print *,'          Output:'
        ij=1
        do i=1,n
          do j=1,n
            if(j>=i) then
              ni(ij)=i
              nj(ij)=j
              print *,ij,i,ni(ij),j,nj(ij),outccf(ij)
              ij=ij+1
            endif
          enddo
        enddo
        print *,' '
!$OMP PARALLEL DO private(i,j,stla1d,stlo1d,stla2d,stlo2d,dist,az,baz,timelen,startp,counter,nptend,ovlp,overlap,nptlen,y1std,y2std,yall,maxv1,maxv2,std1,std2,jj,y1,y2,crt_std,crt1,crt2,yyc,tbeg,tend,k,ij)
        do ij=1,nij
          i=ni(ij)
          j=nj(ij)
c===== work on ACF or CCF
            if(i == j) then
              if(abc == 'A' .or. abc == 'B') then
                goto 90
              else
                print *,'         skip ',outccf(ij) 
                goto 80
              endif
            else ! i/=j
              if(abc == 'B' .or. abc == 'C') then
                goto 90
              else 
                print *,'         skip ',outccf(ij) 
                goto 80
              endif
            endif
c===== check if CCF already exist
90          inquire(file=outccf(ij),exist=alive)                
            if (alive) then
              if (replace == 'Y') then
                goto 70
              else
                print *,'         skip ',outccf(ij) 
                goto 80
              endif
            else
              goto 70
            endif
70          stla1d=DBLE(stla(i))
            stlo1d=DBLE(stlo(i))
            stla2d=DBLE(stla(j))
            stlo2d=DBLE(stlo(j))
            if(stla(i)/=stla(j) .and. stlo(i)/=stlo(j))then
              call elpdst(stla1d,stlo1d,stla2d,stlo2d,dist,az,baz)
            else
              dist=0.0
            endif
c                timelen = int( dist/1.5 + (dist/1.5-dist/4.0)*(2+1) )
c                timelen = int((dist/1+(100/1-100/4)*1.5+(100/1-100/4))*2)
            if(timelen < 600) then
              timelen = 600
            endif
            timelen = 2000

c================================================================
            startp=0
            counter=0
            nptend=np
            ovlp=0.9
            overlap=abs(nint((timelen*(1-ovlp))/dt))
            nptlen=abs(nint((timelen)/dt))
c..... the standard deviation of 2 traces
            y1std = std(yy(i,:),np)
            y2std = std(yy(j,:),np)
c..... initialize the yy arrays
            do k=1,nptlen
              yall(k)=0.0
            enddo
c       print *,ij,'counter',counter,SNGL(dist)
c================================================================
c                if(counter .LE. 1) then
            do while (startp+nptlen.lt.nptend+1)
              maxv1 = 0.0
              maxv2 = 0.0
              std1 = 0.0
              std2 = 0.0
              do k=1,1+nptlen
                jj=k+startp
                y1(k)=yy(i,jj)
                if (y1(k).gt.maxv1) then
                  maxv1 = y1(k)
                endif
                y2(k)=yy(j,jj)
                if (y2(k).gt.maxv2) then
                  maxv2 = y2(k)
                endif
              end do
c================================================================================
c           crt1 = y1std*crt_std
c           crt2 = y2std*crt_std
c           if ((maxv1.lt.crt1).and.(maxv2.lt.crt2)) then
              crt_std = 1.1
              crt1 = std(y1,nptlen)
              crt2 = std(y2,nptlen)
              if ((crt1.lt.y1std*crt_std).and.(crt2.lt.y2std*crt_std)) then
c           if(counter .eq. 0) then
                call ccfunction(y2,y1,dt,nptlen,yyc,timelen,SNGL(dist),counter)
                do k=1,nptlen
                  yall(k)=yall(k)+yyc(k)
                end do
                counter=counter+1
              endif

              startp=startp+overlap
            end do
c            print *,ij,counter,yall(1)
            do k=1,nptlen
              yall(k)=yall(k) / counter
            enddo
            print *,
     %int(ij/real(o)*100),'%       ',outccf(ij),'Dist: ',SNGL(dist),'(km) Time Window:',timelen,'# of stacking:',int(counter),i,stlo(i),j,stlo(j)
c=====================================================================
                tbeg=-dt*(nptlen/2)
                tend=dt*((nptlen-1)/2)
                call NEWHDR
                call setnhv('NPTS',nptlen,nerr)
                call setfhv('DELTA',dt,nerr)
                call setfhv('B',tbeg,nerr)
                call setfhv('E',tend,nerr)
                call setkhv('KSTNM',kstnm(i),nerr)
                call setkhv('KCMPNM',kcmpnm(j),nerr)
                call setfhv('STLA',stla(j),nerr)
                call setfhv('STLO',stlo(j),nerr)
                call setfhv('EVLA',stla(i),nerr)
                call setfhv('EVLO',stlo(i),nerr)
                call wsac0(outccf(ij),xdummy,yall,nerr)
80              continue
        enddo           ! do i=1,n
!$OMP END PARALLEL DO

        do ij=1,nij
          inquire(file=outccf(ij),exist=alive)
          if (alive) then

          i=ni(ij)
          j=nj(ij)
 
          kstnm0=adjustl(kstnm(j))
          lj=len_trim(kstnm0)
          kstnm0=kstnm0(1:lj)

          kstnm1=adjustl(kstnm(i))
          li=len_trim(kstnm1)
          kstnm1=kstnm1(1:li)

          stla1d=DBLE(stla(i))
          stlo1d=DBLE(stlo(i))
          stla2d=DBLE(stla(j))
          stlo2d=DBLE(stlo(j))
          if(stla(i)/=stla(j) .and. stlo(i)/=stlo(j))then
            call elpdst(stla1d,stlo1d,stla2d,stlo2d,dist,az,baz)
          else
            dist=0.0
            az=-12345
            baz=-12345
          endif 

          write(kstnm1(li+1:li+1),'(A1)') '-'
          if(lj == 2 ) write(kstnm1(li+2:li+lj+2),'(A2)') kstnm0
          if(lj == 3 ) write(kstnm1(li+2:li+lj+2),'(A3)') kstnm0
          if(lj == 4 ) write(kstnm1(li+2:li+lj+2),'(A4)') kstnm0
          if(lj == 5 ) write(kstnm1(li+2:li+lj+2),'(A5)') kstnm0
          if(lj == 6 ) write(kstnm1(li+2:li+lj+2),'(A6)') kstnm0
          if(lj == 7 ) write(kstnm1(li+2:li+lj+2),'(A7)') kstnm0
          if(lj == 8 ) write(kstnm1(li+2:li+lj+2),'(A8)') kstnm0

          call rsac1(outccf(ij),yz,np,b,dt,nn,nerr)
          call NEWHDR
          call setnhv('NPTS',np,nerr)
          call setfhv('DELTA',dt,nerr)
          call setfhv('B',b,nerr)
          call setkhv('KEVNM',kstnm1,nerr)
          call setkhv('KCMPNM',kcmpnm(j),nerr)
          call setfhv('STLA',stla(j),nerr)
          call setfhv('STLO',stlo(j),nerr)
          call setfhv('EVLA',stla(i),nerr)
          call setfhv('EVLO',stlo(i),nerr)
          call setfhv('DIST',SNGL(dist),nerr)
          call setfhv('AZ',SNGL(az),nerr)
          call setfhv('BAZ',SNGL(baz),nerr)
    
          call wsac0(outccf(ij),xdummy,yz,nerr)
c        print *,ij,outccf(ij),kstnm1
          endif !if (alive) then
        enddo

        stop
        end
c=======================================================================
        function std(y,nptlen)
c        parameter (nn=1728000)
c        parameter (nn=2**22)
        parameter (nn=172800)
        real y(nn)
        real std,st,sumall,sumsq,avg

        sumall = 0.0
        sumsq = 0.0
        do i=1,nptlen
           sumall = sumall + y(i)
           sumsq = sumsq + y(i)**2
        end do   
        
        avg = sumall / nptlen
        std = sqrt((sumsq-nptlen*(avg**2))/(nptlen-1))
        return
        end
c=======================================================================
        subroutine ccfunction(yy1,yy2,dt1,nptlen,yy,timelen,dist,counter)
c        parameter (nn=32000)                 ! timelen x npts x 4
        parameter (nn=16000)                 ! timelen x npts x 4
        real yy1(nn),yy2(nn),yy(nn)
        complex y1(nn),y2(nn),yw(nn)
        pi=3.14159265358979


	npt=2**ceiling(alog(float(nptlen))/alog(2.))
c..... initialize the y1&y2 complex arrays
        do i=1,nn
          y1(i)=cmplx(0.0,0.0)
          y2(i)=cmplx(0.0,0.0)
        enddo

c..... zero padding at two ends of the trace
100	if(npt.gt.nptlen) then
	  nhead=(npt-nptlen)/2
	  tbeg=0-dt1*nhead
	else
	  nhead=0
	  tbeg=0
	end if

	tail=tbeg+dt1*(npt-1)

c..... 5% cosine taper .....
        ntpr=nptlen*0.05
	scale=pi/ntpr

	do i=1,ntpr
	  yy1(i)=yy1(i)*(0.5-0.5*cos((i-1)*scale))
	  yy1(nptlen-i+1)=yy1(nptlen-i+1)*(0.5-0.5*cos(i*scale))
	  yy2(i)=yy2(i)*(0.5-0.5*cos((i-1)*scale))
	  yy2(nptlen-i+1)=yy2(nptlen-i+1)*(0.5-0.5*cos(i*scale))
	end do

	do i=1,nptlen
	  y1(nhead+i)=cmplx(yy1(i),0.)
	  y2(nhead+i)=cmplx(yy2(i),0.)
	end do
	
c..... do FFT for traces #1 & #2
	call fork(npt,1,y1)
	call fork(npt,1,y2)

c===== cross-correlation in frequency domain
	do i=1,npt
          if (cabs(y1(i)*conjg(y2(i))).eq.0.0) then
             yw(i) = yw(i)
          else
            if(real(y1(1)).eq.real(y2(1)) .and. real(y1(2)).eq.real(y2(2))) then    !ACF ,no whitening
              yw(i)=(y1(i)*conjg(y2(i)))
            else
c            yw(i)=(y1(i)*conjg(y2(i)))   !/(cabs(y1(i)*conjg(y2(i)))) !no whitening
              yw(i)=(y1(i)*conjg(y2(i)))/(cabs(y1(i)*conjg(y2(i))))     !do whitneing
            endif   ! ACF or CCF
          endif
        end do

c===== taper in freqency domain
        if(real(y1(1)).eq.real(y2(1)) .and. real(y1(2)).eq.real(y2(2))) then
          ftaper=2
          ftaper2=8
          ftaper=0
          ftaper2=0
          if(counter.eq.1 .and. ftaper/=0)  print *,'It is ACF, x whitening and apply freq-domain taper in : < ',ftaper,'Hz'
          if(counter.eq.1 .and. ftaper2/=0) print *,'It is ACF, x whitening and apply freq-domain taper in : >',ftaper2,' Hz'
        else 
          ftaper=2*1.5/dist             ! Dist > 2 * Vel * Period
          ftaper2=((npt)/2)/dt1/npt*0.8 !Nyquest Freq *0.8
          ftaper=0
          ftaper2=0
          if(counter.eq.1 .and. ftaper/=0)  print *,'It is CCF, v whitening and apply freq-domain taper in : < ',ftaper,'Hz'
          if(counter.eq.1 .and. ftaper2/=0) print *,'It is CCF, v whitening and apply freq-domain taper in : >',ftaper2,' Hz'
        endif
    
        if (ftaper.NE.0) then
          fnptr=ftaper*dt1*npt
          scale2=pi/fnptr
          do i=1,int(fnptr)
            freq=((i-1)/dt1/npt)
            ratio1i=(0.5-0.5*cos((i-1)*scale2))
            yw(i)=yw(i)*ratio1i**3
          end do
        endif
        if (ftaper2.NE.0) then
          fnptr2=int(ftaper2*dt1*npt) ! npt location of ?? Hz in freq domain
          scale2=pi/(npt/2-fnptr2)
          do i=1,int(npt-fnptr2)
            ii=int(i+fnptr2-1)
            freq=(ii/dt1/npt)
            ratio1i=(0.5+0.5*cos((i-1)*scale2))
            yw(ii)=yw(ii)*ratio1i**3
          end do
        endif !if (ftaper2.NE.0) then

c===== IFFT back to time domain

	call fork(npt,-1,yw)
	do i=1,npt/2
          yy1(i)=real(yw(npt/2+i))
          yy1(i+npt/2)=real(yw(i))
	end do
        do i=1,nptlen
          yy(i)=real(yy1(nhead+i))
        end do

        return
	end


c                 fork.f & fold.f

c===================================================================
c            1-d fft of the function cx(j),j=1,lx
c                     lx
c   cx(k)= SUM (cx(j)* exp {-i_sign*2*pi*i*(j-1)*(k-1)/lx}
c                    j=1

c        point corresponding to Nyquist frequency is lx/2+1

c*** choose your sing convention:  

c    fft: i_sign= 1  yields i;  Fourier Transform
c    fft: i_sign=-1  yields -i; Inverse Fourier Transform

c===================================================================
      subroutine fork (lx,i_sign,cx)
      complex cx(lx),carg,cexp,cw,ctemp
c------------- in inverse fft, a factor of (1/lx) is applied.
      sc=1.
      if(i_sign.lt.0)sc=1./real(lx)
      pi_pni=3.141592654*i_sign

      j=1
      do 30 i=1,lx
      if(i.gt.j)go to 10
      ctemp=cx(j)*sc
      cx(j)=cx(i)*sc
      cx(i)=ctemp
   10 m=lx/2
   20 if(j.le.m)go to 30
      j=j-m
      m=m/2
      if(m.ge.1)go to 20
   30 j=j+m
      l=1
   40 istep=2*l
      do 50 m=1,l
      carg=cmplx(0.,1.)*(pi_pni*(m-1))/real(l)
      cw=cexp(carg)
      do 50 i=m,lx,istep
      ctemp=cw*cx(i+l)
      cx(i+l)=cx(i)-ctemp
   50 cx(i)=cx(i)+ctemp
      l=istep
      if(l.lt.lx)go to 40
      return
      end



c========================================================
c         Nyquist frequency is at nq = lx/2
c       folding across the symmetry axis nq+1=nq1
c   first variable (k=1) folds nowhere, starts from k=2
c========================================================
      subroutine fold(n,nq1,u)
      complex u(1)   
      np2=n+2
      do 820 k=2,nq1
  820 u(np2-k)=conjg(u(k))
      return
      end


        SUBROUTINE ELPDST(EPLAT,EPLON,STLAT,STLON,DIST,AZ,BAZ)
C********************************************************************
C
C Subroutine to calculate the distance between two points
C on an ellipsoidal earth.
C
C Written by P.D. Thomas, NRL, prior to 1973
C Resurected with new flattening etc. by M. Odegard, 1983
C
C********************************************************************
C
        IMPLICIT DOUBLE PRECISION (A-H,P-Z)
C
C Define: The flattening
C 	  The equitorial radius
C 	  pi and radians
C
C WGS84---------
c	F = 298.257223563D+00
c	F = 1.0/F
c	AL = 6378137.00D+00

	F = 298.257223563D+00
	F = 1.0/F
	AL = 6378139.00D+00
	PI2 = 6.28318530717959D+00
	D2R = 1.74532925199433D-02
C
	BOA = 1.0-F
	P1R = EPLAT*D2R
	P2R = STLAT*D2R
	DLR = (STLON-EPLON)*D2R
	T1R = ATAN(BOA*TAN(P1R))
	T2R = ATAN(BOA*TAN(P2R))
	TM = (T1R+T2R)/2.0
	DTM = (T2R-T1R)/2.0
	STM = SIN(TM)
	CTM = COS(TM)
	SDTM = SIN(DTM)
	CDTM = COS(DTM)
	SDLMR = SIN(DLR/2.0)
	TL = SDTM*SDTM+SDLMR*SDLMR*(CDTM*CDTM-STM*STM)
	CD = 1.0-2.0*TL
	DL = ACOS(CD)
	SD = SIN(DL)
	T = DL/SD
	U = 2.0*STM*STM*CDTM*CDTM/(1.0-TL)
	V = 2.0*SDTM*SDTM*CTM*CTM/TL
	D = 4.0*T*T
	X = U+V
	E = -2.0*CD
	Y = U-V
	A = -D*E
	FF64 = F*F/64.0
	DIST = AL*SD*(T-(F/4.0)*(T*X-Y)+FF64*(X*(A+(T-(A+E)/2.0)
     *  *X)+Y*(-2.0*D+E*Y)+D*X*Y))/1000.0
	TDLPM = TAN((DLR+(-((E*(4.0-X)+2.0*Y)*((F/2.0)*T+FF64*
     *  (32.0*T+(A-20.0*T)*X-2.0*(D+2.0)*Y))/4.0)
     *  *TAN(DLR)))/2.0)

	HAPBR = ATAN2(SDTM,CTM*TDLPM)
	HAMBR = ATAN2(CDTM,STM*TDLPM)
	A1M2 = PI2+HAMBR-HAPBR
	A2M1 = PI2-HAMBR-HAPBR
100 	IF((A1M2.GE.0.0D0).AND.(A1M2.LT.PI2)) GOTO 500
	IF(A1M2.GE.PI2) GOTO 400
	A1M2 = A1M2+PI2
	GOTO 100
C
400 	A1M2 = A1M2-PI2
	GOTO 100
C
500 	IF((A2M1.GE.0.0D0).AND.(A2M1.LT.PI2)) GOTO 900
	IF(A2M1.GE.PI2) GOTO 800
	A2M1 = A2M1+PI2
	GOTO 500
C
800 	A2M1 = A2M1-PI2
	GOTO 500
C
900 	AZ = A1M2/D2R
	BAZ = A2M1/D2R
C
	RETURN
C
	END


c--------------------------------------------------------------
        !*******************************************************
        !* Given an array X of N numbers, returns their median *
        !* value XMED. The array X is modified and returned    *
        !* sorted in ascending order.                          *
        !*******************************************************
        SUBROUTINE MDIAN(X,N,XMED)
          real X(N),XMED
          call hpsort(N,X)
          N2=N/2
          if (2*N2.eq.N) then
            XMED = 0.5*(X(N2)+X(N2+1))
          else
            XMED = X(N2+1)
          endif
          return
        END
        !*****************************************************
        !*  Sorts an array RA of length N in ascending order *
        !*                by the Heapsort method             *
        !* ------------------------------------------------- *
        !* INPUTS:                                           *
        !*          N     size of table RA                       *
        !*      RA        table to be sorted                     *
        !* OUTPUT:                                           *
        !*          RA    table sorted in ascending order        *
        !*                                                   *
        !* NOTE: The Heapsort method is a N Log2 N routine,  *
        !*       and can be used for very large arrays.      *
        !*****************************************************
        SUBROUTINE HPSORT(N,RA)
          real RA(N)
          L=N/2+1
          IR=N
          !The index L will be decremented from its initial value during the
          !"hiring" (heap creation) phase. Once it reaches 1, the index IR
          !will be decremented from its initial value down to 1 during the
          !"retirement-and-promotion" (heap selection) phase.
10      continue
          if(L > 1)then
            L=L-1
            RRA=RA(L)
          else
            RRA=RA(IR)
            RA(IR)=RA(1)
            IR=IR-1
            if(IR.eq.1)then
              RA(1)=RRA
              return
            end if
          end if
          I=L
          J=L+L
20        if(J.le.IR)then
          if(J < IR)then
            if(RA(J) < RA(J+1))  J=J+1
          end if
          if(RRA < RA(J))then
            RA(I)=RA(J)
            I=J; J=J+J
          else
            J=IR+1
          end if
          goto 20
          end if
          RA(I)=RRA
          goto 10
        END

        !write table of size N to standard output
        SUBROUTINE TWRIT(N,ARR)
        real ARR(N)
          print *,' '
          WRITE(*,10) (ARR(I),I=1,N)
          return
10        FORMAT(10F6.1)
        END         


         
