#!/bin/csh
#------------------------------------------------------
#
#	12/22/2018 Kai-Xun Chen @ Brown
# 	  for ccf_v3, calculate all the CCFs which daily records are in the same day
#
# 	07/08/2020 Kai-Xun Chen @ Brown
# 	 	The new CCF code is used, which is ccf_v4
# 	  		1. openmp is used to speed up the procedure
# 	 		2. option for ACF, CCF, and both
#
#------------------------------------------------------

#------ environmental parameters for openmp -----------
limit stacksize unlimited
setenv OMP_STACKSIZE   100M
setenv OMP_NUM_THREADS 60		# how many CPUs for openmp
#setenv MALLOC_CHECK_ 3
#------------------------------------------------------
if($#argv != 6) then
  echo 
  echo "zr01_List4CCF.csh [start_year] [start_julian] [end_year] [end_julian] [chn] [ACF(A)/ CCF(C)/ Both(B)]"
  echo  "zr01_List4CCF.csh 2018 1 2018 365 ???  C"
  echo
  exit 1
endif

set oripath = `pwd | awk '{print $0"/D01_SAC"}'`
set pgmccf  = `pwd | awk '{print $0"/ccf_v4"}'`


set chn     = $5
set abc     = $6
# stations lie within this region will be selected
set R1g = (138.75 146 36.9 41.6)
set R1g = (125 132 25 32) 
set R1g = (118.105499      125.706596      19.599701       27.3216)
set R1g = (-361 361 -91 91)
# select stations by users
#set stalist = (TAS KYK OT01)	
set stalist = (all)	# all stations within the selected region
#set stalist = (OT05 OT04)	# all stations within the selected region

set y = $1
while($y <= $3)
  if($y == $1) then
     set j = $2; set jend = 366
  else if ($y != $1 && $y != $3) then
     set j = 1; set jend = 366
  else if ($y == $3) then 
     set j = 1; set jend = $4
  endif
  if($1 == $3) then
    set j = $2; set jend = $4
  endif
   
  while($j <= $jend)
    set jday = `echo $j | awk '{printf "%03d\n",$0}'`
    set out1 = zList1.SingleSta.$y.$jday.inf
    set out2 = zList2.CCF.$y.$jday.inf
    rm -fr $out1 $out2 >& /dev/null

    if(`echo $#stalist` == 1) then
      #saclst stlo stla f $oripath/????/????.$y.$jday.[ZU].SAC $oripath/S1N08/*.$y.$jday.[ZU].SAC | \
      saclst stlo stla f $oripath/*/*.$y.$jday.[ZU].SAC | \
awk '($2>=i && $2<=j && $3>=k && $3<=l){print $0}' i=$R1g[1] j=$R1g[2] k=$R1g[3] l=$R1g[4] | \
sort -k2 -n | awk '{print $1}' | head -n 5  > $out1
    else
      set nr = $#stalist
      #echo $stalist | awk -v RS=" " '{print}' | awk '(NR<=nr){print "saclst stlo stla f "o"/"$0"/"$0"."y"."jday".$chn.SAC"}' nr=$nr o=$oripath y=$y jday=$jday | csh | sort -k2 -n | awk '(NF==3){print $1}' > $out1
      echo $stalist | awk -v RS=" " '{print}' | awk '(NR<=nr){print "saclst stlo stla f "o"/"$0"/"$0"."y"."jday".[ZU].SAC"}' nr=$nr o=$oripath y=$y jday=$jday | csh | sort -k2 -n | awk '(NF==3){print $1}' > $out1
    endif

#    saclst stlo f $oripath/*/*.$y.$jday.[ZU].SAC | sort -k2 -n | awk '{print $1}' > $out1
    set i = 1
    set iT = `wc -l $out1 | awk '{print $1}'`
    while($i<= $iT)
      echo $i
      set k = 1
      while($k <= $iT)
        if($k >= $i) then
          set sta = `awk -F/ '(NR==i || NR==k){print $NF}' i=$i k=$k $out1 | awk -F. '{print $1}'`
          if($k == $i) set sta = ($sta $sta)
          echo "$sta[1]-$sta[2]/$sta[1]-$sta[2].$y.$jday.$chn.CCF" >> $out2
        endif
        @ k++
      end 
      @ i++
    end
    awk -F/ '{print "if (! -d "$1") mkdir "$1}' $out2 | csh
    
#----- run CCF
    time $pgmccf $out1 $out2 $abc Y
    rm -fr $out1 $out2
    @ j++
  end
  @ y++
end

exit 1
  set re1 = "wtl@earth.sinica.edu.tw";set emailtxt = tmp.email
  echo "To: $re1\nCC: $re1\nFrom: dmc<dmc@tecws2.earth.sinica.edu.tw>\nSubject: CHS2020 CCF done" > $emailtxt; cat $emailtxt | sendmail -t
